<?php
require 'vendor/autoload.php';
require 'LatteView.php';
require 'util.php';
require 'DatabaseAuthenticator.php';

/*
 * Set locale based on browser language specification, and set up gettext
 */

$lang = 'en';

if ($locale = set_accept_locale(array('nl'))) {
    putenv("LANG=$locale");
    putenv("LC_ALL=$locale");
    putenv("LC_MESSAGES=$locale");
    $lang = 'nl';
}

$domain = 'archery';
bindtextdomain($domain, __DIR__ . '/locale');
bind_textdomain_codeset($domain, 'UTF-8');
textdomain($domain);

/*
 * Load configs
 */

$config = load_config('config.json');

if (($config_local = load_config('config.local.json', true)) !== null)
    $config = array_merge_recursive($config, $config_local);

if (isset($config['default_timezone']))
    date_default_timezone_set($config['default_timezone']);

define('ROOT_URL', $config['root_url']);

/*
 * Set up app with custom Latte view
 */

$view = new Slim\Latte\LatteView(array(
    'path' => __DIR__ . '/templates',
    'cache' => $config['latte_cache'],
    'configure_engine' => function ($engine) {
        // support for {_'...'} syntax in latte templates
        $engine->addFilter('translate', function() {
            $args = func_get_args();
            assert(count($args) > 0);
            $fmt = gettext(array_shift($args));
            return strpos($fmt, '%') !== false ? vsprintf($fmt, $args) : $fmt;
        });

        // install {form} macro's from Bootstrap form renderer
        $engine->onCompile[] = function ($latte) {
            $compiler = $latte->getCompiler();
            Instante\Bootstrap3Renderer\Latte\FormMacros::install($compiler);
        };
    }
));
$app = new Slim\Slim(compact('view'));

/*
 * Database setup
 */

$connection = new Nette\Database\Connection(
    $config['database']['dsn'],
    $config['database']['user'],
    $config['database']['password'],
    array(
        'lazy' => $config['database']['lazy']
    )
);
$db = new Nette\Database\Context($connection);

/*
 * Security
 */

$factory = new Nette\Http\RequestFactory;
$request = $factory->createHttpRequest();
$response = new Nette\Http\Response;
$session = new Nette\Http\Session($request, $response);
$storage = new Nette\Http\UserStorage($session);
$authenticator = new DatabaseAuthenticator($db);
$user = new Nette\Security\User($storage, $authenticator);

$app->hook('slim.before.router', function () use ($app, $user) {
    $url = $app->request()->getPathInfo();
    $non_user_urls = array('/login', '/register');

    if (!in_array($url, $non_user_urls) && !$user->isLoggedIn())
        $app->redirect(ROOT_URL . '/login');
});

// Extend maximum session lifetime to allow auto-login
$session->setExpiration($config['max_session_lifetime']);

/*
 * Routes
 */

require 'routes/common.php';
require 'routes/register.php';
require 'routes/user.php';
require 'routes/match.php';

// Certain globals should be available in all templates
$view->replace(compact('app', 'config', 'user', 'lang'));
$app->run();
